/***********************************************************************************
 *                                                                                 *
 * Voreen - The Volume Rendering Engine                                            *
 *                                                                                 *
 * Copyright (C) 2005-2019 University of Muenster, Germany,                        *
 * Department of Computer Science.                                                 *
 * For a list of authors please refer to the file "CREDITS.txt".                   *
 *                                                                                 *
 * This file is part of the Voreen software package. Voreen is free software:      *
 * you can redistribute it and/or modify it under the terms of the GNU General     *
 * Public License version 2 as published by the Free Software Foundation.          *
 *                                                                                 *
 * Voreen is distributed in the hope that it will be useful, but WITHOUT ANY       *
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR   *
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.      *
 *                                                                                 *
 * You should have received a copy of the GNU General Public License in the file   *
 * "LICENSE.txt" along with this file. If not, see <http://www.gnu.org/licenses/>. *
 *                                                                                 *
 * For non-commercial academic use see the license exception specified in the file *
 * "LICENSE-academic.txt". To get information about commercial licensing please    *
 * contact the authors.                                                            *
 *                                                                                 *
 ***********************************************************************************/

#include "vesselgraphlistport.h"

namespace voreen {

VesselGraphListPort::VesselGraphListPort(PortDirection direction, const std::string& name, const std::string& guiName, bool allowMultipleConnections, Processor::InvalidationLevel invalidationLevel)
    : GenericPort<std::vector<VesselGraph>>(direction, name, guiName, allowMultipleConnections, invalidationLevel)
{
}

VesselGraphListPort::~VesselGraphListPort() {
}
Port* VesselGraphListPort::create(PortDirection direction, const std::string& id, const std::string& guiName) const
{
    return new VesselGraphListPort(direction,id,guiName);
}
tgt::col3 VesselGraphListPort::getColorHint() const {
    return tgt::col3(64, 128, 0);
}

std::string VesselGraphListPort::getContentDescription() const {
    std::stringstream strstr;
    //port values
    strstr  << Port::getContentDescription();

    if (hasData()) {
        const std::vector<VesselGraph>* graphs = getData();

        strstr << std::endl << graphs->size() << " Graphs";
    }
    return strstr.str();
}

std::string VesselGraphListPort::getContentDescriptionHTML() const {
    return getContentDescription();
}
} // namespace voreen
